-- XLCustom script for Expert Sleepers Augustus Loop 2.1
-- The script was written by Axel Baesler axl99@freenet.de
-- My blog (german): www.ausdemfenster.de
-- Enjoy!

-- uncomment the following for debugging 
--print ( "Augustus Loop 'MultiLoop' script running at ", os.date() )

-- define some local variables
local maxLoop = 4
local currentLoop = 1
local loopState = {0, 0, 0, 0}
local pitchState = {0, 0, 0, 0}
local recTimer
local lengthTime = 0
local stopTime = 0
local FBLevelScale = 100.0/127.0
local levelScale = 0.8/127.0
local FilterScale = 13
local filterTime = 0

-- get the parameter IDs that we want to control
local paramID_ClearLoop 	= getParameterID( "Clear Loop" )
local paramID_TapRecord 	= getParameterID( "Tap Record" )
local paramID_InputLevel	= getParameterID( "Input Level" )
local paramID_FXLevel		= getParameterID( "Effect Level" )
local paramID_FreezeLoop 	= getParameterID( "Freeze Loop" )
local paramID_ReverseLoop 	= getParameterID( "Reverse" )
local paramID_StopLoop	 	= getParameterID( "Stop" )
local paramID_Beats			= getParameterID( "Beats" )
local paramID_Pitch			= getParameterID( "Pitch" )
local paramID_Feedback		= getParameterID( "Master feedback" )
local paramID_FilterOn		= getParameterID( "Filter Type")
local paramID_FilterFreq	= getParameterID( "Filter Cutoff")
local paramID_Saturation	= getParameterID( "Saturation On/Off")

-- If currentLoop becomes 5 the pedals are assigned to
-- special functions applying to all loops
local function selectLoop(channel, cc, value)
	currentLoop = value
end

-- is called automatically when the button 6 is pressed for 1.5 seconds
-- resets the loop as well as its beat number the filter and saturation
local function clearLoop(loop)
	if loop == 1 then
		setOthersParameter( loop, paramID_InputLevel, 0.0 )
		setOthersParameter( loop, paramID_FXLevel, 0.8 )
	else
		setOthersParameter( loop, paramID_FXLevel, 0.0 )
		setOthersParameter( loop, paramID_Feedback, 0.0 )
		setOthersParameter( loop, paramID_InputLevel, 1.0 )
	end
	setOthersParameter( loop, paramID_ClearLoop, 1)
	setOthersParameter( loop, paramID_Saturation, 0 )
	setOthersParameter( loop, paramID_FilterOn, 0 )
	setOthersParameter( loop, paramID_ReverseLoop, 0 )
	setOthersParameter( loop, paramID_Beats, 8 )
	setOthersParameter( loop, paramID_Pitch, 0 )
	setOthersGUIBoolValue( loop, 0, false)
	loopState[loop] = 0
	pitchState[loop] = 0
end

local function clearLoopCallback()
	if currentLoop == 5 then
		for i=1, maxLoop, 1 do
			clearLoop(i)
		end
	else
		clearLoop(currentLoop)
	end
end

-- a noteOn (velocity 100) creates a timer that clears the loop after 1.5 seconds
-- if a noteOff (velocity 0) is received before the 1.5 seconds run out the timer is canceled
local function loopHandler ( channel, noteNumber, velocity )
	if velocity == 100 then
		if loopState[currentLoop] == 0 then			
			if currentLoop == 1 then
				setOthersParameter( currentLoop, paramID_InputLevel, 1.0 )
				setOthersParameter( currentLoop, paramID_TapRecord, 1.0 )
			else
				setOthersParameter( currentLoop, paramID_FXLevel, 0.8 )
				setOthersParameter( currentLoop, paramID_Feedback, 100.0 )
			end
			loopState[currentLoop] = 1
		elseif loopState[currentLoop] == 1 then
			setOthersParameter( currentLoop, paramID_InputLevel, 0.0 )
			if currentLoop == 1 then
				setOthersParameter( currentLoop, paramID_TapRecord, 0.0 )
			end
			loopState[currentLoop] = 2
		elseif loopState[currentLoop] == 2 then
			setOthersParameter( currentLoop, paramID_InputLevel, 1.0 )
			loopState[currentLoop] = 3
		else
			setOthersParameter( currentLoop, paramID_InputLevel, 0.0 )
			loopState[currentLoop] = 2
		end
		recTimer = requestTimedCallback( 1.5, clearLoopCallback )
	else
		cancelTimer(recTimer)
	end
end

-- if button is pressed for less than 1 second, this toggles
-- between FX level 0 and 0.8. Otherwise it reverses the loop
local function stopReverseLoop (channel, noteNumber, velocity)
	if velocity == 100 then
		stopTime = time()
	else
		if (time() - stopTime) > 1 then
			local l = getOthersParameter( currentLoop, paramID_ReverseLoop)
			setOthersParameter( currentLoop, paramID_ReverseLoop, 1 - l )
		else
			local l = getOthersParameter( currentLoop, paramID_FXLevel)
			if l ~= 0.0 then
				setOthersParameter( currentLoop, paramID_FXLevel, 0.0 )
			else
				setOthersParameter( currentLoop, paramID_FXLevel, 0.8 )
			end
		end
	end
end

-- adds beats to the current loop. If the button is pressed for less than 1 second,
-- it adds 1 beat. Otherwise it doubles the length
local function addBeats ( channel, noteNumber, velocity )
	if velocity == 100 then
		lengthTime = time()
	else
		if (time() - lengthTime) > 1 then
			local l = getOthersParameter ( currentLoop, paramID_Beats)
			setOthersParameter( currentLoop, paramID_Beats, l*2 )
		else
			local l = getOthersParameter ( currentLoop, paramID_Beats)
			setOthersParameter( currentLoop, paramID_Beats, l + 1 )		
		end
		lengthTime = 0
	end	
end

-- subtracts beats from the current loop. If the button is pressed for less than 1 second,
-- it subtracts 1 beat. Otherwise it halves the length
local function subtractBeats ( channel, noteNumber, velocity )
	if velocity == 100 then
		lengthTime = time()
	else
		if (time() - lengthTime) > 1 then
			local l = getOthersParameter ( currentLoop, paramID_Beats)
			setOthersParameter( currentLoop, paramID_Beats, l/2 )
		else
			local l = getOthersParameter ( currentLoop, paramID_Beats)
			setOthersParameter( currentLoop, paramID_Beats, l - 1 )		
		end
		lengthTime = 0
	end	
end

-- Freezes the loop and enables the filter. If pressed for less than a second
-- the loop is frozen or unfrozen. Else the filter and saturation are switched on
local function filterHandler ( channel, noteNumber, velocity )
	if velocity == 100 then
		filterTime = time()
	else
		if (time() - filterTime) > 1 then
			local l = getOthersParameter( currentLoop, paramID_FilterOn )
			setOthersParameter( currentLoop, paramID_FilterOn, 1 - l )
			local m = getOthersParameter( currentLoop, paramID_Saturation )
			setOthersParameter( currentLoop, paramID_Saturation, 1 - m )
			setOthersGUIBoolValue( currentLoop, 0, not getOthersGUIBoolValue( currentLoop, 0 ) )		
		else

			local l = getOthersParameter( currentLoop, paramID_FreezeLoop )
			setOthersParameter( currentLoop, paramID_FreezeLoop, 1 - l )
		end
		filterTime = 0
	end
end


-- set Feedback level of selected Loop
local function setFeedbackLevel (channel, cc, value)
	value = value * FBLevelScale
	setOthersParameter (currentLoop, paramID_Feedback, value)
end

-- set global Feedback level
local function setGlobalFeedbackLevel (channel, cc, value)
	value = value * FBLevelScale
	for i=1, maxLoop, 1 do		
		setOthersParameter (i, paramID_Feedback, value)
	end
end

-- set global Volume level
local function setGlobalLevel (channel, cc, value)
	value = value * levelScale
	for i=1, maxLoop, 1 do		
		setOthersParameter (i, paramID_FXLevel, value)
	end
end

-- set Pitch of selected Loop
local function setPitch (channel, cc, value)
	if pitchState[currentLoop] == 0 then
		
		if value == 64 then
			pitchState[currentLoop] = 1
		end
	else
		local minv, maxv = getParameterMinMax( paramID_Pitch )
		if value <= 64 then
			value = value / 128.0
		else
			value = 0.5 + ( value - 64 ) * ( 0.5/63.0 )
		end
		value = minv + value * ( maxv - minv )
		setOthersParameter (currentLoop, paramID_Pitch, value)
	end
end

-- set Filter Cutoff Freq of selected Loop
local function setFilterFreq (channel, cc, value)
	value = 400 + (value * FilterScale)
	setOthersParameter ( currentLoop, paramID_FilterFreq, value)
end

requestNoteOn (1, loopHandler)
requestNoteOff(1, loopHandler)
requestNoteOn (2, filterHandler)
requestNoteOff(2, filterHandler)
requestCC(12, selectLoop)
requestNoteOn(3, stopReverseLoop)
requestNoteOff(3, stopReverseLoop)
requestNoteOn(4, addBeats)
requestNoteOff(4, addBeats)
requestNoteOn(5, subtractBeats)
requestNoteOff(5, subtractBeats)
requestCC(4, setFeedbackLevel)
requestCC(13, setGlobalFeedbackLevel)
requestCC(14, setGlobalLevel)
requestCC(6, setPitch)
--requestCC(6, setFilterFreq)